=begin
#PSI Candidate Rest Eligibility Service

#PSIs REST based service to modify candidate test eligibilities.

OpenAPI spec version: 1.0.0

Generated by: https://github.com/swagger-api/swagger-codegen.git
Swagger Codegen version: 2.4.10

=end

require 'date'

module PsiEligibility
  # An existing eligibility record.
  class ExistingEligibilityModel
    # The last date possible for the candidate to schedule on. Java ISO_OFFSET_DATE_TIME format.
    attr_accessor :eligibility_end_date

    attr_accessor :candidate

    # If required, specific portion codes may be supplied.
    attr_accessor :portion_codes

    # The first date possible for the candidate to schedule on. Java ISO_OFFSET_DATE_TIME format.
    attr_accessor :schedule_start_date

    # Any pertinent message about why a candidate is not eligibile to scheduled yet. Common reasons can be because they have been blocked, have pending special accommodations or other similar reasons. Will be null if candidate is eligible to schedule.
    attr_accessor :eligibility_status_message

    # The client specific test code for the eligibility to be created against.
    attr_accessor :test_code

    # Whether or not this candidate is eligibile to be scheduled.
    attr_accessor :eligible_to_schedule

    # PSI's unique identifier for this eligibility.
    attr_accessor :psi_eligiblity_id

    # Datetime when the eligibility was created. ISO 8601 format similar to  Java ISO_OFFSET_DATE_TIME format.
    attr_accessor :created_datetime

    # Client's unique identifier for this eligibility if provided.
    attr_accessor :client_eligibility_id

    # Only set on this object when returned back to client. Setting this has no effect when provided as part of a create or update. This is the 4 different eligibility statuses available for a candidate: REG01 is Registration Pending, REG02 is Registration approved, REG03 is Cancelled, REG04 is Scheduled
    attr_accessor :status

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'eligibility_end_date' => :'eligibility_end_date',
        :'candidate' => :'candidate',
        :'portion_codes' => :'portion_codes',
        :'schedule_start_date' => :'schedule_start_date',
        :'eligibility_status_message' => :'eligibility_status_message',
        :'test_code' => :'test_code',
        :'eligible_to_schedule' => :'eligible_to_schedule',
        :'psi_eligiblity_id' => :'psi_eligiblity_id',
        :'created_datetime' => :'created_datetime',
        :'client_eligibility_id' => :'client_eligibility_id',
        :'status' => :'status'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'eligibility_end_date' => :'String',
        :'candidate' => :'CandidateModel',
        :'portion_codes' => :'Array<String>',
        :'schedule_start_date' => :'String',
        :'eligibility_status_message' => :'String',
        :'test_code' => :'String',
        :'eligible_to_schedule' => :'BOOLEAN',
        :'psi_eligiblity_id' => :'String',
        :'created_datetime' => :'String',
        :'client_eligibility_id' => :'String',
        :'status' => :'String'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      if attributes.has_key?(:'eligibility_end_date')
        self.eligibility_end_date = attributes[:'eligibility_end_date']
      end

      if attributes.has_key?(:'candidate')
        self.candidate = attributes[:'candidate']
      end

      if attributes.has_key?(:'portion_codes')
        if (value = attributes[:'portion_codes']).is_a?(Array)
          self.portion_codes = value
        end
      end

      if attributes.has_key?(:'schedule_start_date')
        self.schedule_start_date = attributes[:'schedule_start_date']
      end

      if attributes.has_key?(:'eligibility_status_message')
        self.eligibility_status_message = attributes[:'eligibility_status_message']
      end

      if attributes.has_key?(:'test_code')
        self.test_code = attributes[:'test_code']
      end

      if attributes.has_key?(:'eligible_to_schedule')
        self.eligible_to_schedule = attributes[:'eligible_to_schedule']
      end

      if attributes.has_key?(:'psi_eligiblity_id')
        self.psi_eligiblity_id = attributes[:'psi_eligiblity_id']
      end

      if attributes.has_key?(:'created_datetime')
        self.created_datetime = attributes[:'created_datetime']
      end

      if attributes.has_key?(:'client_eligibility_id')
        self.client_eligibility_id = attributes[:'client_eligibility_id']
      end

      if attributes.has_key?(:'status')
        self.status = attributes[:'status']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      status_validator = EnumAttributeValidator.new('String', ['REG01', 'REG02', 'REG03', 'REG04'])
      return false unless status_validator.valid?(@status)
      true
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] status Object to be assigned
    def status=(status)
      validator = EnumAttributeValidator.new('String', ['REG01', 'REG02', 'REG03', 'REG04'])
      unless validator.valid?(status)
        fail ArgumentError, 'invalid value for "status", must be one of #{validator.allowable_values}.'
      end
      @status = status
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          eligibility_end_date == o.eligibility_end_date &&
          candidate == o.candidate &&
          portion_codes == o.portion_codes &&
          schedule_start_date == o.schedule_start_date &&
          eligibility_status_message == o.eligibility_status_message &&
          test_code == o.test_code &&
          eligible_to_schedule == o.eligible_to_schedule &&
          psi_eligiblity_id == o.psi_eligiblity_id &&
          created_datetime == o.created_datetime &&
          client_eligibility_id == o.client_eligibility_id &&
          status == o.status
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [eligibility_end_date, candidate, portion_codes, schedule_start_date, eligibility_status_message, test_code, eligible_to_schedule, psi_eligiblity_id, created_datetime, client_eligibility_id, status].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = PsiEligibility.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
